@@ -5,7 +5,7 @@ use Carp qw/croak confess carp/;
 
 BEGIN {
   $Math::Prime::Util::GMP::AUTHORITY = 'cpan:DANAJ';
-  $Math::Prime::Util::GMP::VERSION = '0.03';
+  $Math::Prime::Util::GMP::VERSION = '0.04';
 }
 
 # parent is cleaner, and in the Perl 5.10.1 / 5.12.0 core, but not earlier.
@@ -14,20 +14,26 @@ use base qw( Exporter );
 our @EXPORT_OK = qw(
                      is_prime
                      is_prob_prime
+                     is_provable_prime
                      is_strong_pseudoprime
                      is_strong_lucas_pseudoprime
                      primes
                      next_prime
                      prev_prime
+                     trial_factor
                      prho_factor
                      pbrent_factor
                      pminus1_factor
                      holf_factor
                      squfof_factor
+                     ecm_factor
                      factor
+                     prime_count
+                     primorial
+                     pn_primorial
+                     consecutive_integer_lcm
                    );
                    # Should add:
-                   # prime_count
                    # nth_prime
 our %EXPORT_TAGS = (all => [ @EXPORT_OK ]);
 
@@ -44,7 +50,8 @@ BEGIN {
 sub _validate_positive_integer {
   my($n, $min, $max) = @_;
   croak "Parameter must be defined" if !defined $n;
-  croak "Parameter '$n' must be a positive integer" if $n =~ tr/0123456789//c;
+  croak "Parameter '$n' must be a positive integer"
+        if $n eq '' || $n =~ tr/0123456789//c;
   croak "Parameter '$n' must be >= $min" if defined $min && $n < $min;
   croak "Parameter '$n' must be <= $max" if defined $max && $n > $max;
   1;
@@ -53,9 +60,10 @@ sub _validate_positive_integer {
 
 sub is_strong_pseudoprime {
   my($n, @bases) = @_;
+  croak "Parameter must be defined" if !defined $n;
   croak "No bases given to is_strong_pseudoprime" unless @bases;
   foreach my $base (@bases) {
-    croak "Base $base is invalid" if $base < 2;
+    _validate_positive_integer($base, 2);
     return 0 unless _GMP_miller_rabin("$n", "$base");
   }
   1;
@@ -117,7 +125,7 @@ Math::Prime::Util::GMP - Utilities related to prime numbers and factoring, using
 
 =head1 VERSION
 
-Version 0.03
+Version 0.04
 
 
 =head1 SYNOPSIS
@@ -129,35 +137,45 @@ Version 0.03
   # enter big number arguments directly as well as enter (e.g.): 2**2048 + 1.
   use bigint;
 
-  # is_prob_prime returns 0 for composite, 2 for prime, and 1 for maybe prime
-  say "$n is ", qw(composite prob_prime def_prime)[is_prob_prime($n)];
+  # These return 0 for composite, 2 for prime, and 1 for probably prime
+  # Numbers under 2^64 will return 0 or 2.
+  # is_prob_prime does a BPSW primality test for numbers > 2^64
+  # is_prime adds a quick test to try to prove the result
+  # is_provable_prime will spend a lot of effort on proving primality
 
-  # is_prime currently is the same -- a BPSW test is used.
-  say "$n is prime" if is_prime($n);
+  say "$n is probably prime"    if is_prob_prime($n);
+  say "$n is ", qw(composite prob_prime def_prime)[is_prime($n)];
+  say "$n is definitely prime"  if is_provable_prime($n) == 2;
 
-  # Run a series of Miller-Rabin tests
+  # Miller-Rabin and strong Lucas-Selfridge pseudoprime tests
   say "$n is a prime or spsp-2/7/61" if is_strong_pseudoprime($n, 2, 7, 61);
-
-  # See if $n is a strong Lucas-Selfridge pseudoprime
-  say "$n is a prime or slpsp" if is_strong_lucas_pseudoprime($n);
+  say "$n is a prime or slpsp"       if is_strong_lucas_pseudoprime($n);
 
   # Return array reference to primes in a range.
   my $aref = primes( 10 ** 200, 10 ** 200 + 10000 );
 
   $next = next_prime($n);    # next prime > n
-
   $prev = prev_prime($n);    # previous prime < n
+
+  # Primorials and lcm
+  say "23# is ", primorial(23);
+  say "The product of the first 47 primes is ", pn_primorial(47);
+  say "lcm(1..1000) is ", consecutive_integer_lcm(1000);
   
+
   # Find prime factors of big numbers
   @factors = factor(5465610891074107968111136514192945634873647594456118359804135903459867604844945580205745718497);
 
   # Finer control over factoring.
   # These stop after finding one factor or exceeding their limit.
-  @factors = prho_factor($n);
-  @factors = pbrent_factor($n);
-  @factors = pminus1_factor($n);
-  @factors = holf_factor($n);
-  @factors = squfof_factor($n);
+  #                               # optional arguments o1, o2, ...
+  @factors = trial_factor($n);    # test up to o1
+  @factors = prho_factor($n);     # no more than o1 rounds
+  @factors = pbrent_factor($n);   # no more than o1 rounds
+  @factors = holf_factor($n);     # no more than o1 rounds
+  @factors = squfof_factor($n);   # no more than o1 rounds
+  @factors = pminus1_factor($n);  # o1 = smoothness limit, o2 = stage 2 limit
+  @factors = ecm_factor($n);      # o1 = B1, o2 = # of curves
 
 =head1 DESCRIPTION
 
@@ -169,11 +187,12 @@ for L<Math::Prime::Util>.  That module will automatically load this if it is
 installed, greatly speeding up many of its operations on big numbers.
 
 Inputs and outputs for big numbers are via strings, so you do not need to
-use a bigint package in your program.  However if you do use bigint, Perl
-will automatically convert input for you, so you do not have to stringify
-your numbers.  This output however will be returned as either Perl scalars
-or strings.  L<Math::Prime::Util> tries to reconvert all strings back into
-the callers bigint type if possible.
+use a bigint package in your program.  However if you do use bigints, inputs
+will be converted internally so there is no need to convert before a call.
+Output results are returned as either Perl scalars (for native-size) or
+strings (for bigints).  L<Math::Prime::Util> tries to reconvert
+all strings back into the callers bigint type if possible, which makes it more
+convenient for calculations.
 
 
 =head1 FUNCTIONS
@@ -187,14 +206,13 @@ Takes a positive number as input and returns back either 0 (composite),
 2 (definitely prime), or 1 (probably prime).
 
 For inputs below C<2^64> a deterministic test is performed, so the possible
-return values are 0 (composite) or 2 (definitely prime).  The current
-implementation uses a strong Baillie-PSW test, but later ones may use
-a deterministic set of Miller-Rabin tests if that is faster for some inputs.
+return values are 0 (composite) or 2 (definitely prime).
 
 For inputs above C<2^64>, a probabilistic test is performed.  Only 0 (composite)
-and 1 (probably prime) are returned.  There is a possibility that composites
-may be returned marked prime, but since the test was published in 1980, not a
-single BPSW pseudoprime has been found, so it is extremely likely to be prime.
+and 1 (probably prime) are returned.  The current implementation uses a strong
+Baillie-PSW test.  There is a possibility that composites may be returned
+marked prime, but since the test was published in 1980, not a single BPSW
+pseudoprime has been found, so it is extremely likely to be prime.
 While we believe (Pomerance 1984) that an infinite number of counterexamples
 exist, there is a weak conjecture (Martin) that none exist under 10000 digits.
 
@@ -203,10 +221,34 @@ exist, there is a weak conjecture (Martin) that none exist under 10000 digits.
 
   say "$n is prime!" if is_prime($n);
 
-This is a misnomer, as the current implementation uses C<is_prob_prime>,
-hence the values are not provably prime if the result is not 2.  Similar to
-the other routine, this takes a single positive number as input and returns
-back either 0 (composite), 2 (definitely prime), or 1 (probably prime).
+Takes a positive number as input and returns back either 0 (composite),
+2 (definitely prime), or 1 (probably prime).  Composites will act exactly
+like C<is_prob_prime>, as will numbers less than C<2^64>.  For numbers
+larger than C<2^64>, some additional tests are performed on probable primes
+to see if they can be proven by another means.
+
+Currently the the method used once numbers have been marked probably
+prime by BPSW is the BLS75 method: Brillhart, Lehmer, and Selfridge's
+improvement to the Pocklington-Lehmer primality test.  The test requires
+factoring C<n-1> to C<(n/2)^(1/3)>, compared to C<n^(1/2)> of the standard
+Pocklington-Lehmer or PPBLS test, or a complete factoring for the Lucas
+test.  The main problem is still finding factors, which is done using
+a small number of rounds of Pollard's Rho.  This works quite well and is
+very fast when the factors are small.
+
+
+=head2 is_provable_prime
+
+  say "$n is definitely prime!" if is_provable_prime($n) == 2;
+
+Takes a positive number as input and returns back either 0 (composite),
+2 (definitely prime), or 1 (probably prime).  A great deal of effort is
+taken to return either 0 or 2 for all numbers.
+
+The current method is the BLS75 algorithm as described in C<is_prime>,
+but using much more aggressive factoring.  Planned enhancements for a later
+release include using a faster method (e.g. APRCL or ECPP), and the ability
+to return a certificate.
 
 
 =head2 is_strong_pseudoprime
@@ -219,7 +261,7 @@ input is a prime or a strong pseudoprime to all of the bases, and 0 if not.
 
 If 0 is returned, then the number really is a composite.  If 1 is returned,
 then it is either a prime or a strong pseudoprime to all the given bases.
-Given enough distinct bases, the chances become very, very strong that the
+Given enough distinct bases, the chances become very strong that the number
 number is actually prime.
 
 Both the input number and the bases may be big integers.  The bases must be
@@ -260,9 +302,9 @@ a lower limit of C<2> if none is given.
 An array reference is returned (with large lists this is much faster and uses
 less memory than returning an array directly).
 
-The current implementation uses repeated calls to C<next_prime>.  This is not
-as efficient as a sieve for large ranges, but also uses no additional memory
-and is fast for very small ranges.
+The current implementation uses repeated calls to C<next_prime>, which is
+good for very small ranges, but not good for large ranges.  A future release
+may use a multi-segmented sieve when appropriate.
 
 
 =head2 next_prime
@@ -280,6 +322,40 @@ Returns the prime smaller than the input number.  0 is returned if the
 input is C<2> or lower.
 
 
+=head2 primorial
+
+  $p = primorial($n);
+
+Given an unsigned integer argument, returns the product of the prime numbers
+which are less than or equal to C<n>.  This definition of C<n#> follows
+L<OEIS series A034386|http://oeis.org/A034386> and
+L<Wikipedia: Primorial definition for natural numbers|http://en.wikipedia.org/wiki/Primorial#Definition_for_natural_numbers>.
+
+=head2 pn_primorial
+
+  $p = pn_primorial($n)
+
+Given an unsigned integer argument, returns the product of the first C<n>
+prime numbers.  This definition of C<p_n#> follows
+L<OEIS series A002110|http://oeis.org/A002110> and
+L<Wikipedia: Primorial definition for prime numbers|http://en.wikipedia.org/wiki/Primorial#Definition_for_prime_numbers>.
+
+The two are related with the relationships:
+
+  pn_primorial($n)  ==   primorial( nth_prime($n) )
+  primorial($n)     ==   pn_primorial( prime_count($n) )
+
+
+=head2 consecutive_integer_lcm
+
+  $lcm = consecutive_integer_lcm($n);
+
+Given an unsigned integer argument, returns the least common multiple of all
+integers from 1 to C<n>.  This can be done by manipulation of the primes up
+to C<n>, resulting in much faster and memory-friendly results than using
+n factorial.
+
+
 =head2 factor
 
   @factors = factor(640552686568398413516426919223357728279912327120302109778516984973296910867431808451611740398561987580967216226094312377767778241368426651540749005659);
@@ -288,23 +364,41 @@ input is C<2> or lower.
 Returns a list of prime factors of a positive number, in numerical order.  The
 special cases of C<n = 0> and C<n = 1> will return C<n>.
 
-The current algorithm uses trial division, then while the number is composite
-it runs a sequence of small Pollard Rho with different functions, small
-Pollard P-1, longer Pollard/Brent Rho, longer Pollard Rho, Pollard P-1 with
-much larger smoothness and a second stage, much longer Pollard/Brent Rho
-trials with different functions, Shanks SQUFOF, and finally will give up.
+Like most advanced factoring programs, a mix of methods is used.  This
+includes trial division for small factors, perfect power detection,
+Pollard's Rho, Pollard's P-1 with various smoothness and stage settings,
+Hart's OLF, and ECM (elliptic curve method).
 
 Certainly improvements could be designed for this algorithm (suggestions are
-welcome).  Most importantly, adding ECM or MPQS/SIQS would make a huge
-difference with larger numbers.  These are non-trivial (though feasible)
-methods.
+welcome).  Most importantly, improving ECM and adding MPQS/SIQS would be a
+big help with larger numbers.  These are non-trivial (though feasible) methods.
 
-In practice, this factors most 26-digit semiprimes in under a second.  Cracking
-14-digit prime factors from large numbers takes about 5 seconds each (Pari
-takes about 1 second, and yafu about 0.3 seconds).  16-digit factors are
-practical but take a long time compared to real factoring programs.  Beyond
-16-digits will take inordinately long.  Note that these are the size of the
-smallest factor, not the size of the input number, as shown by the example.
+In practice, this factors most 26-digit semiprimes in under a second.  It is
+many orders of magnitude faster than any other factoring module on CPAN circa
+2012.  Pari's factorint is faster (and can be accessed from Perl via
+L<Math::Pari>), as are the standalone programs
+L<yafu|http://sourceforge.net/projects/yafu/>,
+L<msieve|http://sourceforge.net/projects/msieve/>,
+L<gmp-ecm|http://ecm.gforge.inria.fr/>,
+L<GGNFS|http://sourceforge.net/projects/ggnfs/>.
+
+
+=head2 trial_factor
+
+  my @factors = trial_factor($n);
+  my @factors = trial_factor($n, 1000);
+
+Given a positive number input, tries to discover a factor using trial division.
+The resulting array will contain either two factors (it succeeded) or the
+original number (no factor was found).  In either case, multiplying @factors
+yields the original input.  An optional divisor limit may be given as the
+second parameter.  Factoring will stop when the input is a prime, one factor
+is found, or the input has been tested for divisibility with all primes less
+than or equal to the limit.  If no limit is given, then C<2**31-1> will be used.
+
+This is a good and fast initial test, and will be very fast for small numbers
+(e.g. under 1 million).  It becomes unreasonably slow in the general case as
+the input size increases.
 
 
 =head2 prho_factor
@@ -312,10 +406,12 @@ smallest factor, not the size of the input number, as shown by the example.
   my @factors = prho_factor($n);
   my @factors = prho_factor($n, 100_000_000);
 
-Produces at most one factor of a positive number input.  An optional number of
-rounds may be given as the second parameter.  Factoring will stop when the
-input is a prime, one factor has been found, or the number of rounds has been
-exceeded.
+Given a positive number input, tries to discover a factor using Pollard's Rho
+method.  The resulting array will contain either two factors (it succeeded)
+or the original number (no factor was found).  In either case, multiplying
+@factors yields the original input.  An optional number of rounds may be
+given as the second parameter.  Factoring will stop when the input is a prime,
+one factor has been found, or the number of rounds has been exceeded.
 
 This is the Pollard Rho method with C<f = x^2 + 3> and default rounds 64M.  It
 is very good at finding small factors.
@@ -326,10 +422,13 @@ is very good at finding small factors.
   my @factors = pbrent_factor($n);
   my @factors = pbrent_factor($n, 100_000_000);
 
-Produces at most one factor of a positive number input.  An optional number of
-rounds may be given as the second parameter.  Factoring will stop when the
-input is a prime, one factor has been found, or the number of rounds has been
-exceeded.
+Given a positive number input, tries to discover a factor using Pollard's Rho
+method with Brent's algorithm.  The resulting array will contain either two
+factors (it succeeded) or the original number (no factor was found).  In
+either case, multiplying @factors yields the original input.  An optional
+number of rounds may be given as the second parameter.  Factoring will stop
+when the input is a prime, one factor has been found, or the number of
+rounds has been exceeded.
 
 This is the Pollard Rho method using Brent's modified cycle detection and
 backtracking.  It is essentially Algorithm P''2 from Brent (1980).  Parameters
@@ -341,35 +440,37 @@ small factors.
 
   my @factors = pminus1_factor($n);
 
-  # Ramp to to B1=10M, with second stages automatically done
+  # Set B1 smoothness to 10M, second stage automatically set.
   my @factors = pminus1_factor($n, 10_000_000);
 
-  # Run p-1 with B1 = 10M, B2 = 100M.  No ramping.
+  # Run p-1 with B1 = 10M, B2 = 100M.
   my @factors = pminus1_factor($n, 10_000_000, 100_000_000);
 
-Produces at most one factor of a positive number input.  An optional maximum
-smoothness factor (B1) may be given as the second parameter in which case the
-algorithm will ramp up to that smoothness factor, also running a second stage.
-If a third parameter (B2) is given, then no ramping happens -- just a first stage
-using the given B1 smoothness followed by a second stage to the B2 smoothness.
+Given a positive number input, tries to discover a factor using Pollard's
+C<p-1> method.  The resulting array will contain either two factors (it
+succeeded) or the original number (no factor was found).  In either case,
+multiplying @factors yields the original input.  An optional first stage
+smoothness factor (B1) may be given as the second parameter.  This will be
+the smoothness limit B1 for for the first stage, and will use C<10*B1> for
+the second stage limit B2.  If a third parameter is given, it will be used
+as the second stage limit B2.
 Factoring will stop when the input is a prime, one factor has been found, or
 the algorithm fails to find a factor with the given smoothness.
 
-This is Pollard's C<p-1> method using a default smoothness of 1M and a
-second stage of C<B2 = 20 * B1>.  It can quickly find a factor C<p> of the input
+This is Pollard's C<p-1> method using a default smoothness of 5M and a
+second stage of C<B2 = 10 * B1>.  It can quickly find a factor C<p> of the input
 C<n> if the number C<p-1> factors into small primes.  For example
 C<n = 22095311209999409685885162322219> has the factor C<p = 3916587618943361>,
 where C<p-1 = 2^7 * 5 * 47 * 59 * 3137 * 703499>, so this method will find
-a factor in the first stage for if C<B1 E<gt>= 703499> or in the second stage if
-C<B2 E<gt>= 703499>.
+a factor in the first stage if C<B1 E<gt>= 703499> or in the second stage if
+C<B1 E<gt>= 3137> and C<B2 E<gt>= 703499>.
 
 The implementation is written from scratch using the basic algorithm including
 a second stage as described in Montgomery 1987.  It is faster than most simple
 implementations I have seen (many of which are written assuming native
-precision inputs), but far slower than Ben Buhrow's code used in earlier
+precision inputs), but slower than Ben Buhrow's code used in earlier
 versions of L<yafu|http://sourceforge.net/projects/yafu/>, and nowhere close
-to the speed of the version included with modern GMP-ECM (as much as 1000x
-slower).
+to the speed of the version included with modern GMP-ECM.
 
 
 
@@ -378,18 +479,23 @@ slower).
   my @factors = holf_factor($n);
   my @factors = holf_factor($n, 100_000_000);
 
-Produces at most one factor of a positive number input.  An optional number of
-rounds may be given as the second parameter.  Factoring will stop when the
-input is a prime, one factor has been found, or the number of rounds has been
-exceeded.
+Given a positive number input, tries to discover a factor using Hart's OLF
+method.  The resulting array will contain either two factors (it succeeded)
+or the original number (no factor was found).  In either case, multiplying
+@factors yields the original input.  An optional number of rounds may be
+given as the second parameter.  Factoring will stop when the input is a
+prime, one factor has been found, or the number of rounds has been exceeded.
 
 This is Hart's One Line Factorization method, which is a variant of Fermat's
 algorithm.  A premultiplier of 480 is used.  It is very good at factoring
 numbers that are close to perfect squares, or small numbers.  Very naive
 methods of picking RSA parameters sometimes yield numbers in this form, so
-it can be useful to run this a few rounds to see.  For example, the number:
+it can be useful to run this a few rounds to check.  For example, the number:
 
-  185486767418172501041516225455805768237366368964328490571098416064672288855543059138404131637447372942151236559829709849969346650897776687202384767704706338162219624578777915220190863619885201763980069247978050169295918863
+  18548676741817250104151622545580576823736636896432849057 \
+  10984160646722888555430591384041316374473729421512365598 \
+  29709849969346650897776687202384767704706338162219624578 \
+  777915220190863619885201763980069247978050169295918863
 
 was proposed by someone as an RSA key.  It is indeed composed of two distinct
 prime numbers of similar bit length.  Most factoring methods will take a
@@ -403,13 +509,16 @@ only a few milliseconds.
   my @factors = squfof_factor($n);
   my @factors = squfof_factor($n, 100_000_000);
 
-Produces at most one factor of a positive number input.  An optional number of
-rounds may be given as the second parameter.  Factoring will stop when the
-input is a prime, one factor has been found, or the number of rounds has been
-exceeded.
+Given a positive number input, tries to discover a factor using Shanks'
+square forms factorization method (usually known as SQUFOF).  The resulting
+array will contain either two factors (it succeeded) or the original number
+(no factor was found).  In either case, multiplying @factors yields the
+original input.  An optional number of rounds may be given as the second
+parameter.  Factoring will stop when the input is a prime, one factor has
+been found, or the number of rounds has been exceeded.
 
 This is Daniel Shanks' SQUFOF (square forms factorization) algorithm.  The
-particular implementation is a non-racing multiple multiplier version, based
+particular implementation is a non-racing multiple-multiplier version, based
 on code ideas of Ben Buhrow and Jason Papadopoulos as well as many others.
 SQUFOF is often the preferred method for small numbers, and L<Math::Prime::Util>
 as well as many other packages use it was the default method for native size
@@ -418,6 +527,29 @@ in this module will work for larger values, but my testing is showing that it
 is not faster than the C<prho> and C<pbrent> methods in general.
 
 
+=head2 ecm_factor
+
+  my @factors = ecm_factor($n);
+  my @factors = ecm_factor($n, 12500);
+  my @factors = ecm_factor($n, 12500, 10);
+
+Given a positive number input, tries to discover a factor using ECM.  The
+resulting array will contain either two factors (it succeeded) or the original
+number (no factor was found).  In either case, multiplying @factors yields the
+original input.  An optional maximum smoothness may be given as the second
+parameter, which relates to the size of factor to search for.  An optional
+third parameter indicates the number of random curves to use at each
+smoothness value being searched.
+
+This is a straightforward implementation of Hendrik Lenstra's elliptic curve
+factoring method, usually referred to as ECM.  Its implementation is textbook,
+with no substantial optimizations done.  It uses a single stage, affine
+coordinates, binary ladder multiplication, and simple initialization.  The
+list of enhancements that can be made is numerous, and it will be much, much
+slower than GMP-ECM.  However, it uses simple GMP and extends the useful
+factoring range of this module.
+
+
 =head1 SEE ALSO
 
 =over 4
@@ -427,19 +559,20 @@ Has many more functions, lots of good code for dealing with native-precision
 arguments (including much faster primes using sieves), and will use this
 module behind the scenes when needed for big numbers.
 
-=item  L<Math::Primality> (version 0.04)
+=item  L<Math::Primality> (version 0.07)
 A Perl module with support for the strong Miller-Rabin test, strong
 Lucas-Selfridge test, the BPSW test, next_prime / prev_prime, and
-prime_count.  It uses L<Math::GMPz> to do all the calculations, so is far
-faster than pure Perl bignums, but somewhat slower than XS+GMP.  The
-prime_count function is only usable for toy-size numbers (it is many
+prime_count.  It uses L<Math::GMPz> to do all the calculations, so is
+faster than pure Perl bignums, but a little slower than XS+GMP.  The
+prime_count function is only usable for very small inputs (it is many
 thousands of times slower than L<Math::Prime::Util>), but the other
-functions are reasonable, though a little slower than this module.
-You'll need a version newer than 0.04 if you use 40+ digit numbers.
+functions are quite reasonable.
+If you use large numbers, make sure to use version 0.05 or newer.
 
 =item L<yafu|http://sourceforge.net/projects/yafu/>, 
 L<msieve|http://sourceforge.net/projects/msieve/>,
-L<gmp-ecm|http://ecm.gforge.inria.fr/>
+L<gmp-ecm|http://ecm.gforge.inria.fr/>,
+L<GGNFS|http://sourceforge.net/projects/ggnfs/>
 Good general purpose factoring utilities.  These will be faster than this
 module, and B<much> faster as the factor increases in size.
 
@@ -452,8 +585,12 @@ module, and B<much> faster as the factor increases in size.
 
 =item Robert Baillie and Samuel S. Wagstaff, Jr., "Lucas Pseudoprimes", Mathematics of Computation, v35 n152, October 1980, pp 1391-1417.  L<http://mpqs.free.fr/LucasPseudoprimes.pdf>
 
+=item John Brillhart, D. H. Lehmer, and J. L. Selfridge, "New Primality Criteria and Factorizations of 2^m +/- 1", Mathematics of Computation, v29, n130, Apr 1975, pp 620-647.  L<http://www.ams.org/journals/mcom/1975-29-130/S0025-5718-1975-0384673-1/S0025-5718-1975-0384673-1.pdf>
+
 =item Richard P. Brent, "An improved Monte Carlo factorization algorithm", BIT 20, 1980, pp. 176-184.  L<http://www.cs.ox.ac.uk/people/richard.brent/pd/rpb051i.pdf>
 
+=item Peter L. Montgomery, "Speeding the Pollard and Elliptic Curve Methods of Factorization", Mathematics of Computation, v48, n177, Jan 1987, pp 243-264.  L<http://www.ams.org/journals/mcom/1987-48-177/S0025-5718-1987-0866113-7/>
+
 =item Richard P. Brent, "Parallel Algorithms for Integer Factorisation", in Number Theory and Cryptography, Cambridge University Press, 1990, pp 26-37.  L<http://www.cs.ox.ac.uk/people/richard.brent/pd/rpb115.pdf>
 
 =item Richard P. Brent, "Some Parallel Algorithms for Integer Factorisation", in Proc. Third Australian Supercomputer Conference, 1999. (Note: there are multiple versions of this paper)  L<http://www.cs.ox.ac.uk/people/richard.brent/pd/rpb193.pdf>
@@ -464,8 +601,6 @@ module, and B<much> faster as the factor increases in size.
 
 =item Jason E. Gower and Samuel S. Wagstaff, Jr, "Square Form Factorization", Mathematics of Computation, v77, 2008, pages 551-588.  L<http://homes.cerias.purdue.edu/~ssw/squfof.pdf>
 
-=item Peter L. Montgomery, "Speeding the Pollard and Elliptic Curve Methods of Factorization", Mathematics of Computation, v48, n177, Jan 1987, pp 243-264.  L<http://www.ams.org/journals/mcom/1987-48-177/S0025-5718-1987-0866113-7/>
-
 =back
 
 
